package awais.instagrabber.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.squareup.picasso.MemoryPolicy;

import awais.instagrabber.R;
import awais.instagrabber.adapters.viewholder.CommentViewHolder;
import awais.instagrabber.models.CommentModel;
import awais.instagrabber.models.ProfileModel;
import awais.instagrabber.utils.MentionClickListener;
import awais.instagrabber.utils.Utils;

public class CommentsAdapter extends RecyclerView.Adapter<CommentViewHolder> {
    private final CommentModel[] commentModels;
    private final boolean isParent;
    private final View.OnClickListener onClickListener;
    private final MentionClickListener mentionClickListener;

    public CommentsAdapter(final CommentModel[] commentModels, final boolean isParent, final View.OnClickListener onClickListener,
                           final MentionClickListener mentionClickListener) {
        this.commentModels = commentModels;
        this.isParent = isParent;
        this.onClickListener = onClickListener;
        this.mentionClickListener = mentionClickListener;
    }

    @NonNull
    @Override
    public CommentViewHolder onCreateViewHolder(@NonNull final ViewGroup parent, final int type) {
        return new CommentViewHolder(LayoutInflater.from(parent.getContext()).inflate(
                isParent ? R.layout.comment_item // parent
                        : R.layout.comment_item_small, // child
                parent, false), onClickListener, mentionClickListener);
    }

    @Override
    public void onBindViewHolder(@NonNull final CommentViewHolder holder, final int position) {
        final CommentModel commentModel = commentModels[position];
        if (commentModel != null) {
            holder.setCommentModel(commentModel);

            holder.setCommment(commentModel.getText());
            holder.setDate(commentModel.getDateTime());

            final ProfileModel profileModel = commentModel.getProfileModel();
            if (profileModel != null) {
                holder.setUsername(profileModel.getUsername());

                Utils.PICASSO.load(profileModel.getSdProfilePic())
                        .memoryPolicy(MemoryPolicy.NO_STORE)
                        .into(holder.getProfilePicView());
            }

            if (holder.isParent()) {
                final CommentModel[] childCommentModels = commentModel.getChildCommentModels();
                if (childCommentModels != null && childCommentModels.length > 0)
                    holder.setChildAdapter(new CommentsAdapter(childCommentModels, false, onClickListener, mentionClickListener));
                else holder.hideChildComments();
            }
        }
    }

    @Override
    public int getItemCount() {
        return commentModels == null ? 0 : commentModels.length;
    }
}