package thoughtbot.expandableadapter;

import androidx.annotation.NonNull;

import thoughtbot.expandableadapter.listeners.ExpandCollapseListener;
import thoughtbot.expandableadapter.models.ExpandableGroup;
import thoughtbot.expandableadapter.models.ExpandableList;
import thoughtbot.expandableadapter.models.ExpandableListPosition;

public class ExpandCollapseController {
    private final ExpandCollapseListener listener;
    private final ExpandableList expandableList;

    public ExpandCollapseController(final ExpandableList expandableList, final ExpandCollapseListener listener) {
        this.expandableList = expandableList;
        this.listener = listener;
    }

    private void collapseGroup(@NonNull final ExpandableListPosition listPosition) {
        expandableList.expandedGroupIndexes[listPosition.groupPos] = false;
        if (listener != null) listener.onGroupCollapsed(expandableList.getFlattenedGroupIndex(listPosition) + 1,
                expandableList.groups.get(listPosition.groupPos).getItemCount());
    }

    private void expandGroup(@NonNull final ExpandableListPosition listPosition) {
        expandableList.expandedGroupIndexes[listPosition.groupPos] = true;
        if (listener != null) listener.onGroupExpanded(expandableList.getFlattenedGroupIndex(listPosition) + 1,
                expandableList.groups.get(listPosition.groupPos).getItemCount());
    }

    public boolean isGroupExpanded(final ExpandableGroup<?> group) {
        return expandableList.expandedGroupIndexes[expandableList.groups.indexOf(group)];
    }

    public void toggleGroup(final int flatPos) {
        final ExpandableListPosition listPos = expandableList.getUnflattenedPosition(flatPos);
        if (expandableList.expandedGroupIndexes[listPos.groupPos]) collapseGroup(listPos);
        else expandGroup(listPos);
    }
}