package thoughtbot.expandableadapter;

import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.List;

import thoughtbot.expandableadapter.listeners.ExpandCollapseListener;
import thoughtbot.expandableadapter.listeners.OnGroupClickListener;
import thoughtbot.expandableadapter.models.ExpandableGroup;
import thoughtbot.expandableadapter.models.ExpandableList;
import thoughtbot.expandableadapter.models.ExpandableListPosition;
import thoughtbot.expandableadapter.viewholders.GroupViewHolder;

public abstract class ExpandableRecyclerViewAdapter<GVH extends GroupViewHolder, CVH extends RecyclerView.ViewHolder>
        extends RecyclerView.Adapter<RecyclerView.ViewHolder> implements ExpandCollapseListener, OnGroupClickListener {
    private final ExpandCollapseController expandCollapseController;
    private final ExpandableList expandableList;

    public ExpandableRecyclerViewAdapter(List<? extends ExpandableGroup<?>> groups) {
        this.expandableList = new ExpandableList(groups);
        this.expandCollapseController = new ExpandCollapseController(expandableList, this);
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull final ViewGroup parent, final int viewType) {
        if (viewType == ExpandableListPosition.GROUP) {
            final GVH gvh = onCreateGroupViewHolder(parent);
            gvh.setOnGroupClickListener(this);
            return gvh;
        }
        return onCreateChildViewHolder(parent);
    }

    @SuppressWarnings("unchecked")
    @Override
    public void onBindViewHolder(@NonNull final RecyclerView.ViewHolder holder, final int position) {
        final ExpandableListPosition listPos = expandableList.getUnflattenedPosition(position);
        final ExpandableGroup<?> group = expandableList.getExpandableGroup(listPos);

        if (listPos.type == ExpandableListPosition.GROUP) {
            final GVH gvh = (GVH) holder;
            onBindGroupViewHolder(gvh, group);
            if (isGroupExpanded(group)) gvh.expand();
            else gvh.collapse();
        } else
            onBindChildViewHolder((CVH) holder, group, listPos.childPos);
    }

    @Override
    public int getItemCount() {
        return expandableList.getVisibleItemCount();
    }

    @Override
    public int getItemViewType(final int position) {
        return expandableList.getUnflattenedPosition(position).type;
    }

    @Override
    public void onGroupExpanded(final int positionStart, final int itemCount) {
        notifyItemChanged(positionStart - 1);
        if (itemCount > 0) notifyItemRangeInserted(positionStart, itemCount);
    }

    @Override
    public void onGroupCollapsed(final int positionStart, final int itemCount) {
        notifyItemChanged(positionStart - 1);
        if (itemCount > 0) notifyItemRangeRemoved(positionStart, itemCount);
    }

    @Override
    public void onGroupClick(final int flatPos) {
        expandCollapseController.toggleGroup(flatPos);
    }

    public void toggleGroup(final int flatPos) {
        expandCollapseController.toggleGroup(flatPos);
    }

    public boolean isGroupExpanded(final ExpandableGroup<?> group) {
        return expandCollapseController.isGroupExpanded(group);
    }

    public abstract GVH onCreateGroupViewHolder(final ViewGroup parent);
    public abstract CVH onCreateChildViewHolder(final ViewGroup parent);
    public abstract void onBindChildViewHolder(final CVH holder, final ExpandableGroup<?> group, final int childIndex);
    public abstract void onBindGroupViewHolder(final GVH holder, final ExpandableGroup<?> group);
}