package thoughtbot.expandableadapter.models;

import androidx.annotation.NonNull;

import java.util.ArrayList;

public class ExpandableListPosition {
    private static final int MAX_POOL_SIZE = 5;
    private static final ArrayList<ExpandableListPosition> sPool = new ArrayList<>(MAX_POOL_SIZE);
    public final static int CHILD = 1;
    public final static int GROUP = 2;
    public int groupPos;
    public int childPos;
    int flatListPos;
    public int type;

    private ExpandableListPosition() { }

    private void resetState() {
        groupPos = 0;
        childPos = 0;
        flatListPos = 0;
        type = 0;
    }

    @NonNull
    public static ExpandableListPosition obtain(final int type, final int groupPos, final int childPos, final int flatListPos) {
        final ExpandableListPosition elp = getRecycledOrCreate();
        elp.type = type;
        elp.groupPos = groupPos;
        elp.childPos = childPos;
        elp.flatListPos = flatListPos;
        return elp;
    }

    @NonNull
    private static ExpandableListPosition getRecycledOrCreate() {
        final ExpandableListPosition elp;
        synchronized (sPool) {
            if (sPool.size() <= 0) return new ExpandableListPosition();
            elp = sPool.remove(0);
        }
        elp.resetState();
        return elp;
    }

    @Override
    public boolean equals(final Object o) {
        if (this == o) return true;
        if (o != null && getClass() == o.getClass()) {
            final ExpandableListPosition that = (ExpandableListPosition) o;
            if (groupPos != that.groupPos) return false;
            if (childPos != that.childPos) return false;
            if (flatListPos != that.flatListPos) return false;
            return type == that.type;
        }
        return false;
    }

    @Override
    public int hashCode() {
        return 31 * (31 * (31 * groupPos + childPos) + flatListPos) + type;
    }
}